#ifndef OBSERVATION_H_
#define OBSERVATION_H_

#include <avr/io.h>
#include <avr/interrupt.h>

#define NOF_TRACE_ENTRIES   (200u)

typedef enum {
    START_EVENT,
    STOP_EVENT
} info_t;

typedef struct {
    info_t         info;
    unsigned short timeStamp;
} event_t;

extern event_t traceBuffer[];
extern unsigned char traceIndex;

void InitObservation(void);

inline unsigned short SafeReadTCNT1(void)
{
    unsigned char  sreg;
    unsigned short retVal;

    sreg   = SREG;  // save interrupt lock status
    cli();          // disable all interrupts
    retVal = TCNT1; // read consistent 16 bit timer value
    SREG   = sreg;  // restore interrupt lock status
    return retVal;
}

inline void traceEvent(info_t info)
{
    unsigned char sreg;
    if (traceIndex < NOF_TRACE_ENTRIES) {
        sreg = SREG; // save interrupt lock status
        cli(); // disable all interrupts
        traceBuffer[traceIndex].timeStamp = SafeReadTCNT1();
        traceBuffer[traceIndex].info = info;
        traceIndex++;
        SREG = sreg; // restore interrupt lock status
    }
}

inline void StartObservation(void)
{
    traceEvent(START_EVENT);
}

inline void StopObservation(void)
{
    traceEvent(STOP_EVENT);
}

#endif /* OBSERVATION_H_ */